@@ -23,18 +23,12 @@ int cap_get_flag(cap_t cap_d, cap_value_t value, cap_flag_t set,
 
     if (raised && good_cap_t(cap_d) && value >= 0 && value < __CAP_BITS
 	&& set >= 0 && set < NUMBER_OF_CAP_SETS) {
-	__cap_s *cap_p = (__cap_s *) (set*CAP_SET_SIZE
-				      + (__u8 *) &cap_d->set);
-
-	*raised = isset_cap(cap_p,value) ? CAP_SET:CAP_CLEAR;
+	*raised = isset_cap(cap_d,value,set) ? CAP_SET:CAP_CLEAR;
 	return 0;
-
     } else {
-
 	_cap_debug("invalid arguments");
 	errno = EINVAL;
 	return -1;
-
     }
 }
 
@@ -43,7 +37,7 @@ int cap_get_flag(cap_t cap_d, cap_value_t value, cap_flag_t set,
  */
 
 int cap_set_flag(cap_t cap_d, cap_flag_t set,
-		 int no_values, cap_value_t *array_values,
+		 int no_values, const cap_value_t *array_values,
 		 cap_flag_value_t raise)
 {
     /*
@@ -60,13 +54,11 @@ int cap_set_flag(cap_t cap_d, cap_flag_t set,
 		_cap_debug("weird capability (%d) - skipped", array_values[i]);
 	    } else {
 		int value = array_values[i];
-		__cap_s *cap_p = (__cap_s *) (set*CAP_SET_SIZE
-					      + (__u8 *) &cap_d->set);
 
 		if (raise == CAP_SET) {
-		    cap_p->raise_cap(value);
+		    cap_d->raise_cap(value,set);
 		} else {
-		    cap_p->lower_cap(value);
+		    cap_d->lower_cap(value,set);
 		}
 	    }
 	}
@@ -89,7 +81,7 @@ int cap_clear(cap_t cap_d)
 {
     if (good_cap_t(cap_d)) {
 
-	memset(&(cap_d->set), 0, sizeof(cap_d->set));
+	memset(&(cap_d->u), 0, sizeof(cap_d->u));
 	return 0;
 
     } else {
@@ -101,3 +93,32 @@ int cap_clear(cap_t cap_d)
     }
 }
 
+/*
+ *  Reset the all of the capability bits for one of the flag sets
+ */
+
+int cap_clear_flag(cap_t cap_d, cap_flag_t flag)
+{
+    switch (flag) {
+    case CAP_EFFECTIVE:
+    case CAP_PERMITTED:
+    case CAP_INHERITABLE:
+	if (good_cap_t(cap_d)) {
+	    unsigned i;
+
+	    for (i=0; i<_LIBCAP_CAPABILITY_U32S; i++) {
+		cap_d->u[i].flat[flag] = 0;
+	    }
+	    return 0;
+	}
+	/*
+	 * fall through
+	 */
+
+    default:
+	_cap_debug("invalid pointer");
+	errno = EINVAL;
+	return -1;
+    }
+}
+
